/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/08/08 18:19:17
 */
import { html, css, Component, classMap, nextTick } from 'wkit'

import { noop } from '../utils/index.js'

class Sidebar extends Component {
  static styles = css`
    :host {
      flex-shrink: 0;
      display: flex;
      flex-direction: column;
      width: 144px;
      height: 100vh;
      background: var(--color-plain-1);
    }
    .noselect {
      -webkit-touch-callout: none;
      -webkit-user-select: none;
      user-select: none;
    }

    .navibar {
      overflow: hidden;
      flex: 1;
      display: flex;
      flex-direction: column;
      width: 100%;
      gap: 8px;
      padding: 16px;
    }

    .navibar .item {
      display: flex;
      justify-content: center;
      align-items: center;
      height: 38px;
      padding: 0 12px;
      gap: 12px;
      border-radius: 20px;
      cursor: pointer;
      transition: background 0.15s ease-in-out;

      --wc-icon-size: 16px;
    }

    .item:hover,
    .item.active {
      background: #fff;
    }
    .item.active {
      color: var(--color-blue-3);
      box-shadow: 0 0 6px rgba(0, 0, 0, 0.05);
    }
  `

  switchNavi(id) {
    let path = '/' + id
    this.$store.navi = id
    if (id === 'containers') {
      path = '/'
    }
    this.$router.push(path)
    localStorage.setItem('last_navi', id)
  }

  mounted() {
    this.switchNavi(this.$store.navi)
  }

  render() {
    return html`
      <menu class="navibar noselect">
        ${this.$store.menus.map(
          it => html`
            <li
              class=${classMap({
                item: true,
                active: it.id === this.$store.navi
              })}
              @click=${ev => this.switchNavi(it.id)}
            >
              <wc-icon name=${it.icon}></wc-icon>
              <span>${it.name}</span>
            </li>
          `
        )}
      </menu>
    `
  }
}

Sidebar.reg('sidebar')
