/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/08/08 18:19:17
 */
import { html, css, Component, classMap, nextTick, outsideClick } from 'wkit'

import { docker } from '../utils/index.js'

class Home extends Component {
  static props = {
    containers: []
  }

  static styles = [
    css`
      :host {
        display: block;
        width: 100%;
        height: 100%;
        color: var(--color-dark-1);
        background: #fff;
      }
      .visible {
        display: block;
      }

      ul li {
        list-style: none;
      }

      .noselect {
        -webkit-touch-callout: none;
        -webkit-user-select: none;
        user-select: none;
      }
    `,

    css`
      .main {
        display: flex;
        flex-direction: column;
        height: 100%;
        padding: 16px;
      }
      .main .toolbar {
        height: 48px;
        border-bottom: 1px solid var(--color-plain-3);
      }
      .main .list {
        overflow: auto;
        flex: 1;
        display: flex;
        flex-direction: column;
        gap: 3px;
        width: 100%;
        height: 100%;
        padding: 12px 0;
      }
      .item,
      .thead {
        flex-shrink: 0;
        display: flex;
        align-items: center;
        gap: 12px;
        width: 100%;
        height: 48px;
        padding: 0 12px;
        line-height: 1.25;
        border-radius: 24px;

        --wc-icon-size: 16px;
      }
      .field {
        overflow: hidden;
        flex: 1;

        &.flex {
          display: flex;
        }

        &.flex-2 {
          flex: 2;
        }

        &.name {
          display: flex;
          flex-direction: column;
        }

        &.center {
          justify-content: center;
          text-align: center;
        }

        &.gap-12 {
          gap: 12px;
        }

        .text-ell {
          overflow: hidden;
          display: block;
          width: 100%;
          white-space: nowrap;
          text-overflow: ellipsis;
          color: var(--color-blue-1);
        }

        .action {
          cursor: pointer;

          &.red {
            color: var(--color-red-1);
          }
          &.green {
            color: var(--color-teal-1);
          }
        }
      }

      .thead {
        color: var(--color-dark-2);
      }
      .item {
        &:hover {
          background: var(--color-plain-1);
        }
      }
    `
  ]

  #input = ''
  #all = true

  mounted() {
    this.#all = +localStorage.getItem('container_mode') === 1
    this.#fetch()
  }

  async #fetch() {
    let list = await docker.containers(this.#all)

    list.forEach(it => {
      it.image = it.image.split(':')
      it.cmd = it.cmd.replace(/^"|"$/, '')
      it.port = it.port
        .replace('0.0.0.0:', '')
        .replace(':::', '')
        .replace(/\/tcp/g, '')
      it.state =
        it.state === 'running' ? '🟢' : it.state === 'created' ? '⚪' : '🔴'
    })

    this.containers = list
  }

  async #filter(ev) {
    let val = ev.currentTarget.value
    if (ev.type === 'input') {
      let txt = val.trim()
      if (this.#input === txt) {
        return
      }
      this.#input = txt
    } else {
      this.#all = val
      localStorage.setItem('container_mode', val ^ 0)
      await this.#fetch()
    }
    this.$requestUpdate()
  }

  async #remove(item) {
    await docker.rm(item.id)
    this.#fetch()
  }

  render() {
    let list = this.containers
    let txt = this.#input

    if (txt) {
      list = list.filter(it => it.name.includes(txt))
    }
    return html`
      <main class="main noselect">
        <wc-space class="toolbar">
          <wc-input
            size="small"
            round
            placeholder="搜索容器"
            @input=${this.#filter}
          ></wc-input>
          <wc-switch :value=${this.#all} @change=${this.#filter}
            >显示非运行中的容器</wc-switch
          >
        </wc-space>

        <ul class="list">
          <li class="thead">
            <span class="field flex-2">容器</span>
            <span class="field flex-2">所属镜像</span>
            <span class="field">命令</span>
            <span class="field">端口</span>
            <span class="field center">状态</span>
            <span class="field center">操作</span>
          </li>
          ${list.map(
            it => html`
              <li class="item">
                <section class="field flex-2 name">
                  <wc-tooltip title=${it.name}>
                    <span class="text-ell">${it.name}</span>
                  </wc-tooltip>
                  <code>${it.id}</code>
                </section>
                <section class="field flex-2">
                  <span class="text-ell">${it.image[0]}</span>
                  <code>${it.image[1]}</code>
                </section>
                <section class="field">
                  <wc-tooltip title=${it.cmd}>
                    <span class="text-ell">${it.cmd}</span>
                  </wc-tooltip>
                </section>
                <section class="field">
                  <wc-tooltip title=${it.port}>
                    <span class="text-ell">${it.port}</span>
                  </wc-tooltip>
                </section>
                <section class="field center">
                  <wc-tooltip title=${it.status}>${it.state}</wc-tooltip>
                </section>
                <section class="field flex gap-12 center">
                  <span class="action ${it.state === '🟢' ? 'red' : 'green'}"
                    >${it.state === '🟢' ? '⏹' : '⏸'}</span
                  >
                  <wc-popconfirm
                    title="是否要删除此容器?"
                    @confirm=${ev => this.#remove(it)}
                  >
                    <span class="action">🗑</span>
                  </wc-popconfirm>
                </section>
              </li>
            `
          )}
        </ul>
      </main>
    `
  }
}

Home.reg('containers')
